/*
 * bg.c
 *
 * Last modified: <2008/12/04 18:10:04 +0900> By SUMIDA
 */
#include <stdio.h>
#include <stdarg.h>
#include <SDL.h>
#include "surface.h"
#include "bg.h"

int BG_MAP_WIDTH;
int BG_MAP_HEIGHT;
BG bg[BG_MAX];

static char *strbuf = NULL;

void bg_init(void) {

	size_t size;
	int i;

	printf("bg_init ... ");

	BG_MAP_WIDTH  = WIDTH/BG_MAP_SIZE;
	BG_MAP_HEIGHT = HEIGHT/BG_MAP_SIZE;
	size = sizeof(MAP_ATTR) * (BG_MAP_WIDTH + BG_MAP_RESERVE) * (BG_MAP_HEIGHT + BG_MAP_RESERVE);
	strbuf = malloc(sizeof(char) * BG_MAP_WIDTH * BG_MAP_HEIGHT);

	for (i = 0; i < BG_MAX; i++) {
		BG *p = &bg[i];
		//p->priority = i * 256 / BG_MAX;
		p->ofs_x   = 8;
		p->ofs_y   = 8;
		p->cnt_x   = 0;
		p->cnt_y   = 0;
		p->font_x  = 0;
		p->font_y  = 0;
		p->frame_x = 1;
		p->frame_y = 1;
		p->frame_w = BG_MAP_WIDTH;
		p->frame_h = BG_MAP_HEIGHT;
		p->scroll_x= 0;
		p->scroll_y= 0;
		p->map = malloc(size);
		memset(bg[i].map, 0, size);
	}

	printf("done.\n");
}

void bg_free(void) {

	int i;

	printf("bg_free ... ");

	free(strbuf);
	strbuf = NULL;

	for (i = 0; i < BG_MAX; i++) {
		BG *p = &bg[i];
		free(p->map);
		p->map = NULL;
	}

	printf("done.\n");
}

void bg_scroll(int bg_no, int dx, int dy) {
	bg[bg_no].ofs_x += dx;
	bg[bg_no].cnt_x += dx;
	bg[bg_no].ofs_y += dy;
	bg[bg_no].cnt_y += dy;
}

void bg_rendaring(SDL_Surface *screen, int bg_no) {

	BG *p = &bg[bg_no];
	int col, row;
	int ofs_x, ofs_y;
	int cmax, rmax;
	int co, ro;

	p->ofs_x = (p->ofs_x + (BG_MAP_WIDTH  + BG_MAP_RESERVE) * BG_MAP_SIZE) % ((BG_MAP_WIDTH  + BG_MAP_RESERVE) * BG_MAP_SIZE);
	p->ofs_y = (p->ofs_y + (BG_MAP_HEIGHT + BG_MAP_RESERVE) * BG_MAP_SIZE) % ((BG_MAP_HEIGHT + BG_MAP_RESERVE) * BG_MAP_SIZE);
	col = p->ofs_x >> 3;
	row = p->ofs_y >> 3;
	ofs_x = (col << 3) - p->ofs_x;
	ofs_y = (row << 3) - p->ofs_y;
	cmax = (ofs_x == 0)? BG_MAP_WIDTH : BG_MAP_WIDTH  + 1;
	rmax = (ofs_y == 0)? BG_MAP_HEIGHT: BG_MAP_HEIGHT + 1;

	for (ro = 0; ro < rmax; ro++) {
		int r = (BG_MAP_HEIGHT + BG_MAP_RESERVE + row + ro) % (BG_MAP_HEIGHT + BG_MAP_RESERVE);
		int y = ofs_y + (ro << 3);
		MAP_ATTR *line = &p->map[r * (BG_MAP_WIDTH + BG_MAP_RESERVE)];

		for (co = 0; co < cmax; co++) {
			int c = (BG_MAP_WIDTH + BG_MAP_RESERVE + col + co) % (BG_MAP_WIDTH + BG_MAP_RESERVE);
			MAP_ATTR *attr = &line[c];

			if (attr->map_id) {
				int x = ofs_x + (co << 3);
				surface_draw_frame8(screen, attr->surface_id, attr->map_id, x, y);
			}
		}
	}
}

void font_frame(int bg_no, int x, int y, int w, int h) {

	BG *p = &bg[bg_no];

	p->frame_x = x;
	p->frame_y = y;
	p->frame_w = w;
	p->frame_h = h;
}

void font_locate(int bg_no, int x, int y) {

	BG *p = &bg[bg_no];

	p->font_x = x;
	p->font_y = y;
}

void font_printf(int bg_no, const char *format, ...) {

	BG *p = &bg[bg_no];

	char *str = strbuf;
	char ch;

	va_list ap;

	va_start(ap, format);

	vsnprintf(strbuf, sizeof(char) * BG_MAP_WIDTH * BG_MAP_HEIGHT, format, ap);

	va_end(ap);

	while (p->font_y < p->frame_h && (ch = *str++) != '\0') {

		//printf("font_x=%d, font_y=%d, frame_x=%d, frame_y=%d\n", p->font_x, p->font_y, p->frame_x, p->frame_y);

		switch (ch) {
		case ' ':
			p->font_x++;
			break;
		case '\t':
			p->font_x += 4;
			break;
		case '\n':
			p->font_x = 0;
			p->font_y++;
			break;
		default:
			p->map[(p->font_x + p->frame_x) + (p->font_y + p->frame_y) * (BG_MAP_WIDTH + BG_MAP_RESERVE)].map_id = ch;
			p->font_x++;
			break;
		}

		if (p->font_x >= p->frame_w) {
			p->font_x = 0;
			p->font_y++;
		}
	}
}
