/*
 * boss1.c
 *
 * Last modified: <2008/10/18 14:41:58 +0900> By SUMIDA
 */
#include <math.h>
#include <SDL.h>
#include "system.h"
#include "texture.h"
#include "ship.h"
#include "shot.h"
#include "enemy.h"
#include "effect.h"

#define SHIP_FLAG_MOVE_MASK    0x0300
#define SHIP_FLAG_MOVE_UP      0x0100
#define SHIP_FLAG_MOVE_DOWN    0x0200

#define SHIP_ACT_MASK          0x00FF
#define SHIP_ACT_MOVE          0x0001
#define SHIP_ACT_RUSH          0x0002
#define SHIP_ACT_BACK          0x0004
#define SHIP_ACT_CHARGE        0x0008
#define SHIP_ACT_DISCHARGE     0x0010

/* NOTE : {X̍s

		SHIP_STAT_APPEAR = o

	SHIP_ACT_MOVE|
	SHIP_FLAG_MOVE_UP/SHIP_FLAG_MOVE_DOWN
		SHIP_STAT_MOVE   = ㉺ړA5waye
		SHIP_STAT_NORMAL = ҋ@
			SHIP_STAT_FIRE = [U[Vbg
	SHIP_ACT_RUSH
			SHIP_STAT_FIRE = ތːi
	SHIP_ACT_BACK
			SHIP_STAT_NORMAL = ޑҋ@

	SHIP_ACT_CHARGE
		SHIP_STAT_MOVE   = @ʈړ
		SHIP_STAT_NORMAL = ҋ@
	SHIP_ACT_DISCHARGE
		SHIP_STAT_FIRE   = [U[
		SHIP_STAT_NORMAL = ҋ@

	ȌAJԂB

	㉺ړ̓_ŕsB
	㉺̓ւ́Aʒ[ɓBB
 */

void boss1_init(SHIP_INFO *info) {

	REPORT(format("info=%p, id=%d", info, info->id));
	system_info.status |= SYSTEM_STAT_BOSS1;
	info->flag |= SHIP_FLAG_NODAMAGE|SHIP_FLAG_HEAVY|SHIP_FLAG_NOCLIP;
	info->count = 240;
	info->sprite.x  = IntToReal(275);
	info->sprite.y  = IntToReal(HEIGHT - 75);
	info->sprite.dx = IntToReal(1);
	info->sprite.dy = IntToReal(0);
}

void boss1(SHIP_INFO *info, SHOT_MGR *shot_mgr, SHIP_INFO *ship) {

	switch (info->stat) {
	case SHIP_STAT_APPEAR:
		REPORT("SHIP_STAT_APPEAR");
		info->sprite.y = IntToReal(HEIGHT - 75);
		if (info->count < 0) {
			info->flag &= ~(SHIP_FLAG_NODAMAGE|SHIP_FLAG_USERS);
			info->stat = SHIP_STAT_MOVE;
			info->count = 60;
			info->sprite.dx = 0;

			/* Nオ */
			info->sprite.dy = ToReal(-2);
			info->flag |= SHIP_ACT_MOVE;
		}
		break;

	case SHIP_STAT_MOVE:
		REPORT("SHIP_STAT_MOVE");
		if (info->count < 0) {
			/* Ԃ玟̃ANV */
			info->stat = SHIP_STAT_NORMAL;
			info->sprite.dx = 0;
			info->sprite.dy = 0;
			info->count = 60 * 2;
		}

		if (info->flag & SHIP_ACT_MOVE) {

			if (info->sprite.dy < 0 && info->sprite.y < IntToReal(36)) {
				/* ʒ[Ŕ] */
				info->flag &= ~SHIP_FLAG_MOVE_MASK;
				info->flag |= SHIP_FLAG_MOVE_DOWN;
				info->sprite.dy = -info->sprite.dy;

			} else if (info->sprite.dy > 0 && info->sprite.y > IntToReal(HEIGHT - 75)) {
				/* ʒ[Ŕ] */
				info->flag &= ~SHIP_FLAG_MOVE_MASK;
				info->flag |= SHIP_FLAG_MOVE_UP;
				info->sprite.dy = -info->sprite.dy;
			}

		} else {

			SURFACE *surface_info = surface_get(info->sprite.surface_id);
			SURFACE *surface_ship = surface_get(ship->sprite.surface_id);

			Real y0 = info->sprite.x + (IntToReal(surface_info->frame_height)>>1);
			Real y1 = ship->sprite.x + (IntToReal(surface_ship->frame_height)>>1);
			Real dy = y1 - y0;

			if (-8 > dy || dy > 8) {
				/* @ɕύX */
				info->sprite.dy = -info->sprite.dy;
			} else {
				/* ̃ANV */
				info->sprite.dx = 0;
				info->sprite.dy = 0;
				info->count = 60 * 1;
				info->stat = SHIP_STAT_NORMAL;
			}
		}

		/* 5waye */
		if (Random(0, 256) < system_info.level) {

			/* @Zo */
			double angle = sprite_get_angle(&info->sprite, &ship->sprite);
			SHOT_INFO *shot1 = shot_new(shot_mgr, SHOT_ID_BALLET);
			SHOT_INFO *shot2 = shot_new(shot_mgr, SHOT_ID_BALLET);
			SHOT_INFO *shot3 = shot_new(shot_mgr, SHOT_ID_BALLET);
			SHOT_INFO *shot4 = shot_new(shot_mgr, SHOT_ID_BALLET);
			SHOT_INFO *shot5 = shot_new(shot_mgr, SHOT_ID_BALLET);

			if (shot1 != NULL) {
				shot_fire(shot1, info, 0, 0);
				sprite_set_angle(&shot1->sprite, angle, IntToReal(2));
			}
			if (shot2 != NULL) {
				shot_fire(shot2, info, 0, 0);
				sprite_set_angle(&shot2->sprite, angle - 15, IntToReal(2));
			}
			if (shot3 != NULL) {
				shot_fire(shot3, info, 0, 0);
				sprite_set_angle(&shot3->sprite, angle + 15, IntToReal(2));
			}
			if (shot4 != NULL) {
				shot_fire(shot4, info, 0, 0);
				sprite_set_angle(&shot4->sprite, angle - 30, IntToReal(2));
			}
			if (shot5 != NULL) {
				shot_fire(shot5, info, 0, 0);
				sprite_set_angle(&shot5->sprite, angle + 30, IntToReal(2));
			}
		}
		break;

	case SHIP_STAT_NORMAL:
		REPORT("SHIP_STAT_NORMAL");
		if (info->count < 0) {
			info->count = 5;

			if (info->flag & (SHIP_ACT_BACK|SHIP_ACT_DISCHARGE)) {

				if (info->sprite.x < IntToReal(-16)) {

					int rnd = (int)Random(-3, 3);

					info->flag &= ~SHIP_FLAG_USERS;
					info->stat = SHIP_STAT_MOVE;
					info->sprite.dx = 0;

					if (rnd < 0) {
						info->flag |= SHIP_ACT_MOVE|SHIP_FLAG_MOVE_UP;
						info->sprite.dy = -IntToReal(2);
						info->count = 120;

					} else if (rnd > 0) {
						info->flag |= SHIP_ACT_MOVE|SHIP_FLAG_MOVE_DOWN;
						info->sprite.dy = IntToReal(2);
						info->count = 120;

					} else {
						info->flag |= SHIP_ACT_CHARGE;
					}
				}

			} else if (info->flag & SHIP_ACT_CHARGE) {

				info->stat = SHIP_STAT_FIRE;
				info->flag &= ~SHIP_ACT_MASK;
				info->flag |= SHIP_ACT_DISCHARGE;
				info->sprite.dx = -IntToReal(1)>>1;

			} else {

				info->stat = SHIP_STAT_FIRE;

				/* [U[VbgƓːȋI */
				if (system_info.level > 7 && Random(0, 4) < 1) {
					info->flag &= ~SHIP_ACT_MASK;
					info->flag |= SHIP_ACT_RUSH;
					info->sprite.dx = -IntToReal(1)>>1;
				}
			}
		}
		break;

	case SHIP_STAT_FIRE:
		REPORT("SHIP_STAT_FIRE");
		if (info->count < 0) {

			info->count = 5;

			if (info->flag & SHIP_ACT_RUSH) {

				if (info->sprite.dx < 0 && info->sprite.x < -IntToReal(64)) {
					info->sprite.dx = IntToReal(4);
					//info->sprite.dy = 0;

				} else if (info->sprite.dx > 0 && info->sprite.x > IntToReal(WIDTH - 16)) {
					info->flag &= ~SHIP_ACT_MASK;
					info->flag |= SHIP_ACT_BACK;
					info->stat = SHIP_STAT_NORMAL;
					info->count = 60 * 1;
					info->sprite.dx = -IntToReal(1);
					//nfo->sprite.dy = 0;
				}

			} else if (info->flag & SHIP_ACT_DISCHARGE) {

				/* [U[̔ */
				SHOT_INFO *shot = shot_new(shot_mgr, SHOT_ID_RAY2);

				if (shot != NULL) {
					shot->sprite.x = info->sprite.x + IntToReal(176);
					shot->sprite.y = info->sprite.y + IntToReal(58);
					shot->sprite.dx = IntToReal(3);
				}

				if (Random(-2, 18) > system_info.level) {
					info->stat = SHIP_STAT_NORMAL;
					info->count = 60 * 2;
					info->sprite.dx = 0;
				}

			} else {
				if (Random(0, 16) < system_info.level) {

					/* [U[Vbg̔ */
					SHOT_INFO *shot1 = shot_new(shot_mgr, SHOT_ID_RAY1);
					SHOT_INFO *shot2 = shot_new(shot_mgr, SHOT_ID_RAY1);
					SHOT_INFO *shot3 = shot_new(shot_mgr, SHOT_ID_RAY1);
					SHOT_INFO *shot4 = shot_new(shot_mgr, SHOT_ID_RAY1);

					if (shot1 != NULL) {
						shot1->sprite.x = info->sprite.x + IntToReal(176);
						shot1->sprite.y = info->sprite.y + IntToReal(59);
						shot1->sprite.dx = ToReal(2.5);
					}

					if (shot2 != NULL) {
						shot2->sprite.x = info->sprite.x + IntToReal(176);
						shot2->sprite.y = info->sprite.y + IntToReal(62);
						shot2->sprite.dx = ToReal(2.5);
					}

					if (shot3 != NULL) {
						shot3->sprite.x = info->sprite.x + IntToReal(145);
						shot3->sprite.y = info->sprite.y + IntToReal(25);
						shot3->sprite.dx = ToReal(2.5);
					}

					if (shot4 != NULL) {
						shot4->sprite.x = info->sprite.x + IntToReal(156);
						shot4->sprite.y = info->sprite.y + IntToReal(42);
						shot4->sprite.dx = ToReal(2.5);
					}
				}

				if (Random(-4, 18) > system_info.level) {
					info->stat = SHIP_STAT_MOVE;
					info->count = 60;
					info->sprite.dx = 0;

					/* tړ */
					if (info->flag & SHIP_FLAG_MOVE_UP) {
						info->sprite.dy = IntToReal(2);
						info->flag &= ~SHIP_FLAG_MOVE_MASK;
						info->flag |= SHIP_ACT_MOVE|SHIP_FLAG_MOVE_DOWN;
					} else {
						info->sprite.dy = -IntToReal(2);
						info->flag &= ~SHIP_FLAG_MOVE_MASK;
						info->flag |= SHIP_ACT_MOVE|SHIP_FLAG_MOVE_UP;
					}
				}
			}
		}
		break;

	default:
		break;
	}
}

void boss1_term(SHIP_INFO *info) {

	REPORT(format("info=%p", info));
	system_info.status &= ~SYSTEM_STAT_BOSS1;
	enemy_explosion();
}
