/*
 * boss2_horn.c
 *
 * Last modified: <2008/10/18 10:30:29 +0900> By SUMIDA
 */
#include <SDL.h>
#include "system.h"
#include "texture.h"
#include "ship.h"
#include "shot.h"
#include "enemy.h"
#include "effect.h"
#include "sound.h"

#define HORN_ALIVE1 0x0100
#define HORN_ALIVE2 0x0200
#define HORN_ALIVE3 0x0400
#define HORN_ALIVE4 0x0800

#define HORN_ALIVE  0x0f00

typedef struct HORN_INFO {
	Real x;
	Real y;
	int flag;
} HORN_INFO;

static struct HORN_INFO hone_info[] = {
	{IntToReal(64) , IntToReal(185), HORN_ALIVE1},
	{IntToReal(173), IntToReal(209), HORN_ALIVE2},
	{IntToReal(255), IntToReal(211), HORN_ALIVE3},
	{IntToReal(279), IntToReal(176), HORN_ALIVE4},
};

void boss2_horn_init(SHIP_INFO *info) {

	HORN_INFO *horn = &hone_info[info->id - SHIP_ID_BOSS2_HORN1];
	SHIP_INFO *parent = info->parent;

	REPORT(format("info=%p, id=%d", info, info->id));

	info->flag |= SHIP_FLAG_NOCLIP|SHIP_FLAG_HEAVY;

	if (info->id != SHIP_ID_BOSS2_HORN4) {
		info->flag |= SHIP_FLAG_NODAMAGE;
	}

	if (parent != NULL) {
		info->sprite.x = parent->sprite.x + horn->x;
		info->sprite.y = parent->sprite.y + horn->y;
		parent->flag |= horn->flag;
	}
}

void boss2_horn(SHIP_INFO *info, SHOT_MGR *shot_mgr, SHIP_INFO *ship) {

	HORN_INFO *horn = &hone_info[info->id - SHIP_ID_BOSS2_HORN1];
	SHIP_INFO *parent = info->parent;

	switch (info->stat) {
	case SHIP_STAT_EXPLOSION:
		{
			EFFECT_INFO *effect = effect_new(EFFECT_ID_EXPLOSION);
			effect_set_pos(effect, &info->sprite, 0, 0);
			sound_effect(SE_BOMB0);
		}
		break;

	case SHIP_STAT_LOST:
		if (parent != NULL) {
			parent->flag &= ~horn->flag;
			if (!(parent->flag & HORN_ALIVE)) {
				parent->flag &= ~(SHIP_FLAG_NODAMAGE|SHIP_FLAG_HEAVY);
			}
		}
		break;

	default:
		if (parent != NULL) {
			info->sprite.x = parent->sprite.x + horn->x;
			info->sprite.y = parent->sprite.y + horn->y;

			if (info->id != SHIP_ID_BOSS2_HORN4
				&& !(parent->flag & hone_info[info->id - SHIP_ID_BOSS2_HORN1 + 1].flag)) {
				info->flag &= ~SHIP_FLAG_NODAMAGE;
			}

			if (parent->stat == SHIP_STAT_EXPLOSION) {
				EFFECT_INFO *effect = effect_new(EFFECT_ID_EXPLOSION);
				effect_set_pos(effect, &info->sprite, 0, 0);
				sound_effect(SE_BOMB0);
				info->stat = SHIP_STAT_EXPLOSION;
				info->count = SHIP_DEF_LOST_WAIT;
				info->sprite.options &= ~SPRITE_USE_MOVE;
			}
		}
		break;
	}
}

void boss2_horn_term(SHIP_INFO *info) {
	REPORT(format("info=%p", info));
}

