/*
 * input.c
 *
 * Last modified: <2008/06/28 05:45:24 +0900> By SUMIDA
 */
#include <SDL.h>
#include "system.h"
#include "input.h"

enum {
	JOYSTICK_BUTTON1 = 2,
	JOYSTICK_BUTTON2 = 3,
	JOYSTICK_SELECT  = 1,
	JOYSTICK_START   = 0
};

Uint32 input[PLAYER_MAX];
RealVector axis[PLAYER_MAX][2];

static SDL_Joystick *joystick[PLAYER_MAX];
static int num_joysticks = 0;

void input_init(void) {
	int i;

	printf("input_init ... {\n");
	memset(input, 0, NUMBEROF(input));
	memset(joystick, NULL, NUMBEROF(joystick));
	memset(axis, 0, NUMBEROF(axis));

	SDL_JoystickEventState(SDL_IGNORE);
	num_joysticks = SDL_NumJoysticks();

	if (num_joysticks >= PLAYER_MAX) num_joysticks = PLAYER_MAX;

	for (i = 0; i < num_joysticks; i++) {
		printf("\tjoystick%d: [%s]", i, SDL_JoystickName(i));
		joystick[i] = SDL_JoystickOpen(i);
		if (joystick[i] == NULL) {
			printf(" ... failure!\n");
		} else {
			SDL_Joystick *p = joystick[i];
			printf(" buttons=%d, hats=%d, axes=%d, balls=%d\n",
				   SDL_JoystickNumButtons(p),
				   SDL_JoystickNumHats(p),
				   SDL_JoystickNumAxes(p),
				   SDL_JoystickNumBalls(p));
		}
	}
	printf("} done.\n");
}

void input_free(void) {

	int i;

	printf("input_free ... ");
	for (i = 0; i < num_joysticks; i++) {
		SDL_Joystick *p = joystick[i];

		if (p != NULL && SDL_JoystickOpened(i)) {
			SDL_JoystickClose(p);
		}
	}
	printf("done.\n");
}

Uint32 joystick_read(SDL_Joystick *p, RealVector *vec) {

	Uint32 stat = 0;

	Uint32 hat = SDL_JoystickGetHat(p, 0);

	if (hat & SDL_HAT_LEFT ) stat |= INPUT_PRESS_LEFT;
	if (hat & SDL_HAT_UP   ) stat |= INPUT_PRESS_UP;
	if (hat & SDL_HAT_RIGHT) stat |= INPUT_PRESS_RIGHT;
	if (hat & SDL_HAT_DOWN ) stat |= INPUT_PRESS_DOWN;
	if (SDL_JoystickGetButton(p, JOYSTICK_BUTTON1)) stat |= INPUT_PRESS_BUTTON1;
	if (SDL_JoystickGetButton(p, JOYSTICK_BUTTON2)) stat |= INPUT_PRESS_BUTTON2;
	if (SDL_JoystickGetButton(p, JOYSTICK_SELECT )) stat |= INPUT_PRESS_SELECT;
	if (SDL_JoystickGetButton(p, JOYSTICK_START  )) stat |= INPUT_PRESS_START;

	vec[0].x = (Sint32)SDL_JoystickGetAxis(p, 0) << 8;
	vec[0].y = (Sint32)SDL_JoystickGetAxis(p, 1) << 8;
	vec[1].x = (Sint32)SDL_JoystickGetAxis(p, 2) << 8;
	vec[1].y = (Sint32)SDL_JoystickGetAxis(p, 3) << 8;

	return stat;
}

void input_read(void) {

	static Uint32 count = 0;
	Uint8* keys = SDL_GetKeyState(NULL);
	int i;

	++count;

	SDL_JoystickUpdate();

	for (i = 0; i < PLAYER_MAX; i++) {
		Uint32 prev_input = input[i] & INPUT_PRESS_MASK;
		Uint32 curr_input = 0;
		SDL_Joystick *p = joystick[i];

		input[i] = 0;

		if (i == P1) {
			/* ݂̏Ԃ擾 */
			if (keys[SDLK_LEFT] | keys[SDLK_h])
				curr_input |= INPUT_PRESS_LEFT;          /* [], [H] */
			if (keys[SDLK_UP] | keys[SDLK_k])
				curr_input |= INPUT_PRESS_UP;            /* [], [K] */
			if (keys[SDLK_RIGHT] | keys[SDLK_l])
				curr_input |= INPUT_PRESS_RIGHT;         /* [], [L] */
			if (keys[SDLK_DOWN] | keys[SDLK_j])
				curr_input |= INPUT_PRESS_DOWN;          /* [], [J] */
			if (keys[SDLK_LSHIFT] | keys[SDLK_z])
				curr_input |= INPUT_PRESS_BUTTON1;       /* Shift, [Z] */
			if (keys[SDLK_LCTRL] | keys[SDLK_x])
				curr_input |= INPUT_PRESS_BUTTON2;       /* Ctrl, [X] */
			if (keys[SDLK_a])
				curr_input |= INPUT_PRESS_SELECT;        /* [A] */
			if (keys[SDLK_s])
				curr_input |= INPUT_PRESS_START;         /* [S] */
		}

		/* WCXeBbN̏Ԃǂݎ */
		if(p != NULL && SDL_JoystickOpened(i)) {
			curr_input |= joystick_read(p, axis[i]);
		}

		/* KEY PRESS */
		input[i] = curr_input;

		/* KEY DOWN */
		input[i] |= (curr_input & ~prev_input) << 8;

		/* KEY UP */
		input[i] |= (~curr_input & prev_input) << 16;

		/* KEY REPEAT */
		if (count & 0x01) input[i] |= curr_input << 24;
	}
}
