/*
 * map.c
 *
 * Last modified: <2008/10/18 14:55:16 +0900> By SUMIDA
 */
#include <SDL.h>
#include "system.h"
#include "real.h"
#include "surface.h"
#include "sprite.h"
#include "ship.h"
#include "shot.h"
#include "enemy.h"
#include "map.h"
#include "input.h"
#include "sound.h"
#include "effect.h"

#define STAT_READY     0x8000
#define STAT_EMPTY     0x1000

#define STAT_TASKS     0x0FFF
#define TASK_BGM1      0x0001
#define TASK_BGM2      0x0002

typedef struct MAP_INFO {

	Uint16 id;       /* CxgID */
	Sint16 count;    /* o     */
	Sint16 interval; /* oԊu   */
	Uint16 status;   /* }bv */

} MAP_INFO;

/* TODO : VjG@̏o߂B */
static MAP_INFO map[] = {
	{  -1, 1,    0, STAT_EMPTY|TASK_BGM1},
	{  -1, 1, 60*2, STAT_EMPTY},
	{SHIP_ID_ENEMY1,  5, 20, 0},
	{SHIP_ID_ENEMY2,  3, 20, 0},
	{SHIP_ID_ENEMY3,  2, 20, 0},
	{SHIP_ID_ENEMY4,  2, 20, 0},
	{SHIP_ID_ENEMY5,  2, 20, 0},
	{SHIP_ID_ENEMY9,  4, 20, 0},
	{SHIP_ID_ENEMY2,  2, 20, 0},
	{SHIP_ID_ENEMY3,  2, 20, 0},
	{SHIP_ID_ENEMY4,  2, 20, 0},
	{SHIP_ID_ENEMY5,  2, 20, 0},
	{  -1, 1,   60, STAT_EMPTY|TASK_BGM2},
	{SHIP_ID_BOSS1 ,  1, 20, 0},
	{  -1, 1,    0, STAT_EMPTY|TASK_BGM1},
	{  -1, 1, 60*2, STAT_EMPTY},
	{SHIP_ID_ENEMY1, 10, 20, 0},
	{SHIP_ID_ENEMY2, 10, 20, 0},
	{SHIP_ID_ENEMY4,  5, 20, 0},
	{SHIP_ID_ENEMY5,  3, 20, 0},
	{SHIP_ID_ENEMY6,  2, 20, 0},
	{SHIP_ID_ENEMY2,  5, 20, 0},
	{SHIP_ID_ENEMY8,  1, 20, 0},
	{SHIP_ID_ENEMY4,  3, 20, 0},
	{SHIP_ID_ENEMY7,  2, 20, 0},
	{SHIP_ID_ENEMY5,  2, 20, 0},
	{SHIP_ID_ENEMY9,  5, 20, 0},
	{  -1, 1,   60, STAT_EMPTY|TASK_BGM2},
	{SHIP_ID_BOSS2 ,  1, 20, 0},
	{  -1, 1,    0, STAT_EMPTY},
};

extern SHIP_MGR enemy_mgr;

static MAP_INFO curr_map;
static void map_action_bgm1(void);
static void map_action_bgm2(void);

static void(*func_action[])(void) = {
	NULL,
	map_action_bgm1,
	map_action_bgm2,
};

void map_init(void) {
	system_info.level = (system_info.options & SYSTEM_OPT_LEVEL)?
		system_info.options & SYSTEM_OPT_LEVEL_TYPE: SYSTEM_OPT_EASY;
	system_info.targets   = 0;
	system_info.encounter = 0;
	system_info.map_index = 0;
	memset(&curr_map, 0, sizeof(curr_map));
}

void map_action(void) {
	/* TODO : TjG@oB */
#if 0
	if (input[P1] & INPUT_KEYDOWN_SELECT) {
		//SHIP_INFO *enemy = enemy_new(&enemy_mgr, SHIP_ID_BOSS2, 0, 0);
		SHIP_INFO *enemy = enemy_new(&enemy_mgr, SHIP_ID_ENEMY7, 0, 0);
		//SHIP_INFO *enemy = enemy_new(&enemy_mgr, (int)Random(SHIP_ID_ENEMY1, SHIP_ID_MAX), 0, 0);

		if (enemy != NULL) {
			SURFACE *surface = surface_get(enemy->sprite.surface_id);

			enemy->sprite.x = IntToReal(-surface->frame_width);
			enemy->sprite.y = IntToReal((int)Random(0, HEIGHT - surface->frame_height));
		}
	}
#endif

	if (!(system_info.frame & 0xff)) {
		/* ACeELA̎ */
		SHIP_INFO *enemy = enemy_new(&enemy_mgr, SHIP_ID_BOX, 0, 0);

		if (enemy != NULL) {
			SURFACE *surface = surface_get(enemy->sprite.surface_id);

			enemy->sprite.x = IntToReal(-surface->frame_width);
			enemy->sprite.y = IntToReal((int)Random(0, HEIGHT - surface->frame_height));
			//enemy->sprite.y = IntToReal(10);
		}
	}

	if (system_info.status & SYSTEM_STAT_BOSS) return;

	if (system_info.encounter > 0) {
		--system_info.encounter;
		return;
	}

	if (!(curr_map.status & STAT_READY)) {
		memcpy(&curr_map, &map[system_info.map_index], sizeof(curr_map));
		curr_map.status |= STAT_READY;
	}

	/* TODO : TjG@oB */
#if 1  /* <== Ӂ UjG@̓ ɍƂsӏI */
	if (--curr_map.interval < 0) {
		if (!(curr_map.status & STAT_EMPTY)) {
			/* G@̐ */
			SHIP_INFO *enemy = enemy_new(&enemy_mgr, curr_map.id, 0, 0);

			if (enemy != NULL) {
				SURFACE *surface = surface_get(enemy->sprite.surface_id);

				enemy->sprite.x = IntToReal(-surface->frame_width);
				enemy->sprite.y = IntToReal((int)Random(0, HEIGHT / surface->frame_height) * surface->frame_height);
				enemy->sprite.dy = IntToReal((int)Random(-2, 2));
			}

		} else {
			int task = curr_map.status & STAT_TASKS;

			if (task < NUMBEROF(func_action) && func_action[task] != NULL) {
				func_action[task]();
			}

			curr_map.status &= ~STAT_TASKS;
		}

		if (--curr_map.count > 0) {
			curr_map.interval = map[system_info.map_index].interval;
		} else {
			system_info.encounter = 60;
			curr_map.status &= ~STAT_READY;
			if (++system_info.map_index >= NUMBEROF(map)) {
				system_info.map_index = 0;
				if (++system_info.level > LEVEL_MAX) system_info.level = LEVEL_MAX;
			}
		}
	}
#endif
}

void map_action_bgm1(void) {

	sound_music(BGM_01, -1);
}

void map_action_bgm2(void) {

	sound_music(BGM_02, -1);
}
/*
void map_action_moon(void) {

	EFFECT_INFO *effect = effect_new(EFFECT_ID_MOON);

	if (effect != NULL) {
		SURFACE *surface = surface_get(effect->sprite.surface_id);

		effect->sprite.x = IntToReal((int)Random(0, WIDTH / surface->frame_width) * surface->frame_width);
		effect->sprite.y = IntToReal(-surface->frame_height);
		effect->sprite.dx = 0;
		effect->sprite.dy = 0x1000;
	}
}

void map_action_planet(void) {

	EFFECT_INFO *effect = effect_new(EFFECT_ID_PLANET);

	if (effect != NULL) {
		SURFACE *surface = surface_get(effect->sprite.surface_id);

		effect->sprite.x = IntToReal((int)Random(0, WIDTH / surface->frame_width) * surface->frame_width);
		effect->sprite.y = IntToReal(-surface->frame_height);
		effect->sprite.dx = 0;
		effect->sprite.dy = 0x2000;
	}
}
*/
