/*
 * myship.c
 *
 * Last modified: <2008/08/30 01:03:37 +0900> By SUMIDA
 */
#include <stdlib.h>
#include <math.h>
#include <SDL.h>
//#include <SDL_image.h>
#include "system.h"
#include "myship.h"
#include "item.h"
#include "input.h"
#include "effect.h"
#include "sound.h"

#define SIDE_SPACE   48
#define BOTTOM_SPACE 30

/* @̏ */
void myship_init(MYSHIP_MGR *mgr) {

	mgr->ship = ship_new(mgr->ship_mgr, SHIP_ID_ME);
	mgr->ship->sprite.x = IntToReal(WIDTH     - 100);
	mgr->ship->sprite.y = IntToReal(HEIGHT/ 2 - 100);
	memset(&mgr->ability, 0, sizeof(mgr->ability));

	/*mgr->ship->sprite.options &= ~SPRITE_USE_COLLISION; /**/
}

/* @̐ */
void myship_action(MYSHIP_MGR *mgr) {

	SHOT_MGR *shot_mgr = mgr->shot_mgr;
	SHIP_INFO *ship    = mgr->ship;

	Real speed;

	/* \͎gp`FbN */
	if (ship->flag & ITEM_FLAG_POWERUP) {
		mgr->ability.powerup--;
		if (mgr->ability.powerup <= 0) {
			ship->flag &= ~ITEM_FLAG_POWERUP;
		}
	}
	if (ship->flag & ITEM_FLAG_SPEEDUP) {
		mgr->ability.speedup--;
		if (mgr->ability.speedup <= 0) {
			ship->flag &= ~ITEM_FLAG_SPEEDUP;
		}
	}
	if (ship->flag & ITEM_FLAG_MISSILE) {
		mgr->ability.missile--;
		if (mgr->ability.missile <= 0) {
			ship->flag &= ~ITEM_FLAG_MISSILE;
		}
	}
	if (ship->flag & ITEM_FLAG_3WAY) {
		mgr->ability.threeway--;
		if (mgr->ability.threeway <= 0) {
			ship->flag &= ~ITEM_FLAG_3WAY;
		}
	}

	/* TODO : Sj@𓮂B */
	speed = (ship->flag & ITEM_FLAG_SPEEDUP) ? ToReal(3): ToReal(2);
	/* @̑ */
	if (input[P1] & INPUT_PRESS_LEFT ) ship->sprite.x -= speed; /* */
	if (input[P1] & INPUT_PRESS_RIGHT) ship->sprite.x += speed; /* */
	if (input[P1] & INPUT_PRESS_UP   ) ship->sprite.y -= speed; /* */
	if (input[P1] & INPUT_PRESS_DOWN ) ship->sprite.y += speed; /* */

	if (ship->sprite.x < IntToReal(-SIDE_SPACE))           ship->sprite.x = IntToReal(-SIDE_SPACE);
	if (ship->sprite.x > IntToReal(WIDTH - SIDE_SPACE))    ship->sprite.x = IntToReal(WIDTH - SIDE_SPACE);
	if (ship->sprite.y < 0)                                ship->sprite.y = 0;
	if (ship->sprite.y > IntToReal(HEIGHT - BOTTOM_SPACE)) ship->sprite.y = IntToReal(HEIGHT - BOTTOM_SPACE);

	if ((input[P1] & INPUT_REPEAT_BUTTON1)
		&& ship->stat != SHIP_STAT_LOST) {

		int id = (ship->flag & ITEM_FLAG_POWERUP) ? SHOT_ID_NORMAL2: SHOT_ID_NORMAL;
		SHOT_INFO *shot = shot_new(shot_mgr, id);

		if (shot != NULL) {
			shot_fire(shot, ship, IntToReal(-14), IntToReal(-28));
			shot->sprite.dx = IntToReal(-16);
		}

		if (ship->flag & ITEM_FLAG_3WAY) {
			/*SHOT_INFO *shot2 = shot_new(mgr->shot2_mgr, id);/**/
			/*SHOT_INFO *shot3 = shot_new(mgr->shot3_mgr, id);/**/
			SHOT_INFO *shot2 = shot_new(mgr->shot2_mgr, SHOT_ID_NORMAL);/**/
			SHOT_INFO *shot3 = shot_new(mgr->shot3_mgr, SHOT_ID_NORMAL);/**/

			if (shot2 != NULL) {
				shot_fire(shot2, ship, IntToReal(-14), IntToReal(-28));
				shot2->sprite.dx = IntToReal(-16);
				shot2->sprite.dy = IntToReal(-16);
			}
			if (shot3 != NULL) {
				shot_fire(shot3, ship, IntToReal(-14), IntToReal(-28));
				shot3->sprite.dx = IntToReal(-16);
				shot3->sprite.dy = IntToReal(16);
			}
		}
	}

	if ((input[P1] & INPUT_KEYDOWN_BUTTON1)
		&& ship->stat != SHIP_STAT_LOST
		&& (ship->flag & ITEM_FLAG_MISSILE)) {

		SHOT_INFO *shot = shot_new(mgr->missile_mgr, SHOT_ID_MISSILE);

		if (shot != NULL) {
			shot_fire(shot, ship, IntToReal(-14), IntToReal(20));
			shot->sprite.dx = IntToReal(4);
			shot->sprite.dy = ToReal(Random(-4, 4));
		}
	}

	/* XvCg̐ */
	sprite_action(&ship->sprite);

	/* Ԃɉo */
	switch (ship->stat) {
	case SHIP_STAT_EXPLOSION:
		if (ship->count == 0) {
			sound_stop_effect();
		}
		sound_effect(SE_BOMB2);
		if (ship->count++ > 2) {
			ship->stat = SHIP_STAT_LOST;
			system_info.status = SYSTEM_MODE_OVER;
			sound_music(BGM_03, 1);
		}
		break;

	case SHIP_STAT_LOST:
		if (!sound_playing_music()) {
			system_info.status = SYSTEM_MODE_READY;
			sound_music(BGM_SCRAMBLE, -1);
		}
		break;

	default:
		if (--ship->count < 0) {
			EFFECT_INFO *effect = effect_new(EFFECT_ID_STARS2);

			if (effect != NULL) {
				effect_set_pos(effect, &ship->sprite, IntToReal(80), IntToReal(-2));
				effect->sprite.dx = ToReal(Random(1, 2));
				effect->sprite.dy = ToReal(Random(1, 2));
				ship->count = 10;
			}
		}
		break;
	}

	/* TODO : Rj@\B */
	/* XvCg̕\ */
	if (ship->stat != SHIP_STAT_LOST) {
		sprite_put(&ship->sprite);
	}
}
